import ru.ifmo.testlib.Checker;
import ru.ifmo.testlib.CheckerFramework;
import ru.ifmo.testlib.InStream;
import ru.ifmo.testlib.Outcome;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Objects;
import java.util.Set;
import java.util.function.Predicate;
import java.util.regex.Pattern;
import java.util.stream.Stream;

import static ru.ifmo.testlib.Outcome.Type.*;

/**
 * @author Georgiy Korneev (kgeorgiy@kgeorgiy.info)
 */
public class Check implements Checker {
    private static final int MEMORY_SIZE = 1 << 12;
    private static final int IO_WORD = MEMORY_SIZE - 1;
    private static final int STOP_IP = MEMORY_SIZE - 2;
    private static final int MAX_STEPS = 1 << 12;
    private static final Predicate<String> WORD
            = Pattern.compile("[0-9a-f]{4}", Pattern.CASE_INSENSITIVE).asMatchPredicate();

    public Outcome test(final InStream inf, final InStream ouf, final InStream ans) {
        final int x = inf.nextInt();

        final int[] memory = new int[MEMORY_SIZE];
        int address = 0;
        while (!ouf.seekEoF()) {
            final String token = ouf.nextToken();
            if (!WORD.test(token)) {
                return new Outcome(PE, String.format("Invalid %d-th word: '%s'", address, token));
            }
            if (address == IO_WORD) {
                return new Outcome(PE, "Program too long");
            }
            memory[address++] = Integer.parseInt(token, 16);
        }

        int maxSteps = 0;
        for (int i = 0; i < (1 << 16); i++) {
            maxSteps = Math.max(maxSteps, test(x, i, memory.clone()));
        }
        return new Outcome(OK, String.format("%d words, %s steps", address, maxSteps));
    }

    private int test(final int x, final int v, final int[] memory) {
        memory[IO_WORD] = v;
        int ip = 0;
        int steps = 0;
        while (ip < STOP_IP) {
            if (steps > MAX_STEPS) {
                throw new Outcome(WA, String.format("%d: Too many steps for %d", x, v));
            }

            final int a = memory[ip];
            final int bit = (memory[a >> 4] >> (a & 15)) & 1;
            final int bWord = memory[ip + 1] >> 4;
            final int bBit = memory[ip + 1] & 15;
            memory[bWord] = memory[bWord] & ~(1 << bBit) | bit << bBit;
            ip = memory[ip + 2];
            steps++;
        }

        final int expected = x == v ? 1 : 0;
        final int actual = memory[IO_WORD] & 1;
        if (actual != expected) {
            throw new Outcome(WA, String.format("%d: Expected %d, found %d for %d", x, expected, actual, v));
        }

        return steps;
    }

    public static void main(final String[] args) {
        CheckerFramework.main(args);
    }
}
