#include <bits/stdc++.h>

using namespace std;

#include <ext/pb_ds/assoc_container.hpp>
#include <ext/pb_ds/tree_policy.hpp>
using namespace __gnu_pbds;

template <typename T>
using ordered_set = tree<T, null_type, less_equal<T>, rb_tree_tag, tree_order_statistics_node_update>;

#define CE constexpr

#ifdef DBG
#define debugf(s, ...) fprintf(stderr, "(" #__VA_ARGS__ ") " s "\n", __VA_ARGS__)
#else
#define debugf(...) void(0)
#endif

using i64 = int64_t;
using i128 = __int128;


constexpr int ninf = -1e9;

struct edge {
    int u, v;
    int idx;
    int next = -1;
    int prev = -1;
    int mark = -1;
    int ans = ninf;
};


ostream &operator<<(ostream &out, const edge &e) {
    return out << '(' << e.u << ',' << e.v << ',' << e.idx << ',' << e.next << ',' << e.prev << ',' << e.mark << ',' << e.ans << ')';
}


void solve() {
    int n;

    int m;
    cin >> n >> m;
    vector<edge> edges;

    vector<set<int>> g(n);

    for (int i = 0; i < m; ++i) {
        int u, v;
        cin >> u >> v;
        g[--u].insert(i);
        g[--v].insert(i);
        edges.emplace_back(u, v, i);
    }

    auto cg = g;

    if (m % 2 == 1) {
        cout << "NO\n";
        return;
    }

    vector<int> last_left(n, -1);

    vector<int> euler;

    auto dfs = [&](auto &&self, int u) -> void {
        while (!g[u].empty()) {
            auto it = g[u].begin();
            auto &e = edges[*it];
            if (e.u != u)
                swap(e.u, e.v);
            int v = e.v;
            g[u].erase(e.idx);
            g[v].erase(e.idx);
            last_left[u] = e.idx;
            if (last_left[v] != -1) {
                e.next = last_left[v];
                edges[last_left[v]].prev = e.idx;
            }

            self(self, v);
            euler.push_back(e.idx);
        }
    };

    dfs(dfs, 0);

    for (int i = 0; i < m; ++i) {
        edges[euler[i]].mark = i % 2;
    }

    for (int i = 0; i < n; ++i) {
        auto enter = ranges::find_if(cg[i], [&](int x) {return edges[x].v == i && edges[x].next == -1;});
        auto leave = ranges::find_if(cg[i], [&](int x) {return edges[x].u == i && edges[x].prev == -1;});

        if (enter != cg[i].end()) {
            edges[*enter].next = *leave;
            edges[*leave].prev = *enter;
        }
    }


    for (int i = 0; i < m; ++i) {
        if (edges[i].ans != ninf) continue;

        int num_2 = 0;
        int num_1 = 0;

        int it = i;
        int nxt = edges[i].next;

        do {
            num_2 += edges[it].mark == edges[nxt].mark;
            num_1 += edges[it].mark != edges[nxt].mark;
            it = nxt;
            nxt = edges[nxt].next;
        } while (it != i);

        int cnt_2 = 0, cnt_1 = 0;
        int lim_2, lim_1;
        if (num_2 % 2 == 0) {
            lim_2 = num_2 / 2;
            lim_1 = num_1 / 2;
        } else {
            lim_2 = num_2 / 2 + 1;
            lim_1 = num_1 / 2 - 1;
        }

        edges[it].ans = 0;

        do {
            if (edges[it].mark == edges[nxt].mark) {
                if (cnt_2++ < lim_2) {
                    edges[nxt].ans = edges[it].ans + 2;
                } else {
                    edges[nxt].ans = edges[it].ans - 2;
                }
            } else {
                if (cnt_1++ < lim_1) {
                    edges[nxt].ans = edges[it].ans + 1;
                } else {
                    edges[nxt].ans = edges[it].ans - 1;
                }
            }
            it = nxt;
            nxt = edges[nxt].next;
        } while (it != i);
    }

    vector<bool> finalised(n);

    auto dfs_final = [&](auto &&self, edge &e, int shift) -> void {
        e.ans += shift;
        int v = e.v;
        if (finalised[v]) return;
        finalised[v] = true;
        auto &ne = edges[e.next];
        self(self, ne, shift);
        int val_for_2, val_for_1;
        if (e.mark == ne.mark) {
            val_for_2 = min(e.ans, ne.ans) + 1;
            val_for_1 = val_for_2 + 3;
        } else {
            val_for_2 = ninf;
            val_for_1 = min(e.ans, ne.ans) + 2;
        }

        for (auto idx: cg[v]) {
            auto &pe = edges[idx];
            if (pe.u != v) continue;
            if (pe.idx == ne.idx) continue;

            auto &ppe = edges[pe.prev];
            int min_ans = min(ppe.ans, pe.ans);
            if (ppe.mark == pe.mark) {
                if (val_for_2 == ninf) {
                    self(self, pe, val_for_1 - min_ans);
                    val_for_2 = val_for_1 + 1;
                    val_for_1 += 4;
                } else {
                    self(self, pe, val_for_2 - min_ans);
                    val_for_2 = ninf;
                }
            } else {
                self(self, pe, val_for_1 - min_ans);
                val_for_1 += 2;
            }
        }
    };

//    for (auto &e: edges) {
//        cout << e.mark << ' ';
//    }
//    cout << '\n';
//
//    for (auto &e: edges) {
//        cout << e.ans << ' ';
//    }
//    cout << '\n';

    dfs_final(dfs_final, edges[0], 0);

    int mn = -ninf;
    int mx = ninf;

    for (auto &e: edges) {
        mn = min(e.ans, mn);
        mx = max(e.ans, mx);
    }

    cout << "YES\n";
    cout << mx - mn + 1 << '\n';
    for (auto &e: edges) {
        cout << e.ans - mn + 1 << ' ';
    }
    cout << '\n';

}


int main() {
#ifndef DBG
    std::ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
#endif

    int t = 1;

//    cin >> t;
    for (int i = 0; i < t; ++i) { solve(); }
    return 0;
}