#include <iostream>
#include <vector>
#include <unordered_map>
#include <functional>
#include <algorithm>


// Input data:
int n;
std::vector<int> op1, op2, op3, op4;
std::vector<int> operations;
int m;
std::vector<std::pair<int, int>> edges;

void read_input() {
    std::cin >> n;
    for (int i = 0; i < n; ++i) {
        std::string op_type, op_result;
        std::cin >> op_type >> op_result;

        if (op_type == "set" && op_result == "true") {
            op1.push_back(i);
            operations.push_back(1);

        } else if (op_type == "unset" && op_result == "true") {
            op2.push_back(i);
            operations.push_back(2);

        } else if (op_type == "set" && op_result == "false") {
            op3.push_back(i);
            operations.push_back(3);

        } else if (op_type == "unset" && op_result == "false") {
            op4.push_back(i);
            operations.push_back(4);
        }
    }

    std::cin >> m;

    for (int i = 0; i < m; ++i) {
        int a, b;
        std::cin >> a >> b;
        edges.emplace_back(a - 1, b - 1);
    }
}

std::unordered_map<int, std::vector<int>> build_dag() {
    std::unordered_map<int, std::vector<int>> dag;

    for (auto [a, b]: edges) {
        dag[a].push_back(b);

        // (1, 4) || (3, 4) => (2, 4)
        if ((operations[a] == 1 || operations[a] == 3) && operations[b] == 4) {
            if (op2.empty()) {
                // No solution!
                dag[a].push_back(a);
                return dag;
            }
            dag[op2[0]].push_back(b);
        }

        // (4, 2) || (4, 3) => (4, 1)
        if (operations[a] == 4 && (operations[b] == 2 || operations[b] == 3)) {
            if (op1.empty()) {
                // No solution!
                dag[a].push_back(a);
                return dag;
            }
            dag[a].push_back(op1[0]);
        }
    }

    for (int i: op1) {
        for (int j: op3) {
            dag[i].push_back(j);
        }
    }
    for (int i: op3) {
        for (int j: op2) {
            dag[i].push_back(j);
        }
    }
    for (int i: op1) {
        for (int j: op2) {
            dag[i].push_back(j);
        }
    }
    return dag;
}

std::optional<std::vector<int>> top_sort(
        int n,
        std::unordered_map<int, std::vector<int>> &dag,
        std::vector<int> const &op1
) {
    std::vector<int> result;
    std::vector<bool> gray(n, false);
    std::vector<bool> black(n, false);
    std::function<bool(int)> dfs;

    dfs = [&](int v){
        if (black[v]) {
            return false;
        }
        if (gray[v]) {
            return true;
        }
        gray[v] = true;
        for (auto u: dag[v]) {
            if (dfs(u)) {
                return true;
            }
        }

        result.push_back(v);
        black[v] = true;
        return false;
    };

    for (int v: op1) {
        if (dfs(v)) {
            return std::nullopt;
        }
    }
    for (int i = 0; i < n; ++i) {
        if (dfs(i)) {
            return std::nullopt;
        }
    }

    std::reverse(result.begin(), result.end());
    return result;
}

int main() {
    read_input();

    if (op1.empty() && (!op2.empty() || !op3.empty())) {
        std::cout << -1 << " ";
        return 0;
    }

    auto dag = build_dag();

    auto top_sort_order = top_sort(n, dag, op1);

    if (top_sort_order == std::nullopt) {
        std::cout << -1 << std::endl;
    } else {
        for (int v: *top_sort_order) {
            std::cout << (v + 1) << " ";
        }
        std::cout << std::endl;
    }
    return 0;
}
