#include "testlib.h"
#include <optional>

using namespace std;

const char* correct = "Correct";
const char* impossible = "Impossible";

typedef long long number;
const number MAX_NUMBER = 9999999999LL;

struct Parser {
    string& s;
    int i;
    int len;

    Parser(string &_s) : s(_s), i(0), len(s.length()) {}

    optional<number> nextNumber() {
        number result = 0;
        int j = i;
        while (j < len && s[j] >= '0' && s[j] <= '9') {
            result = result * 10 + (s[j] - '0');
            if (result > MAX_NUMBER) return nullopt;
            j++;
        }
        if (j == i) return nullopt;
        if (result == 0 && j != i + 1) return nullopt;
        if (result != 0 && s[i] == '0') return nullopt;
        i = j;
        return result;
    }

    optional<number> nextExpression() {
        optional<number> next = nextNumber();
        if (!next) return nullopt;
        number result = next.value();
        while (i < len && (s[i] == '+' || s[i] == '-')) {
            char op = s[i];
            i++;
            next = nextNumber();
            if (!next) return nullopt;
            switch (op) {
                case '+': result += next.value(); break;
                case '-': result -= next.value(); break;
            }
        }
        return result;
    }

    optional<pair<number, number>> nextEquality() {
        optional<number> left = nextExpression();
        if (!left) return nullopt;
        if (i >= len || s[i] != '=') return nullopt;
        i++;
        optional<number> right = nextExpression();
        if (!right) return nullopt;
        if (i != len) return nullopt;
        return make_pair(left.value(), right.value());
    }
};

bool isWellformedEquality(string &s) {
    return Parser(s).nextEquality().has_value();
}

bool isCorrectEquality(string& s) {
    optional<pair<number, number>> eq = Parser(s).nextEquality();
    if (!eq) return false;
    return eq->first == eq->second;
}

bool isMovingDigit(string& s, string& t) {
    for (int i = 0; i < s.length(); i++) {
        if (s[i] >= '0' && s[i] <= '9') {
            string t1{ s.substr(0, i) };
            t1.append(s.substr(i + 1));
            for (int j = 0; j < s.length(); j++) {
                string t2{ t1.substr(0, j) };
                t2.push_back(s[i]);
                t2.append(t1.substr(j));
                if (t2 == t) {
                    return true;
                }
            }
        }
    }
    return false;
}

pattern basicExpressionPattern{ "[0-9+\\-]+=[0-9+\\-]+" };
pattern basicOutputPattern{ basicExpressionPattern.src() + "|" + correct + "|" + impossible };

int main(int argc, char* argv[]) {
    registerTestlibCmd(argc, argv);
    string in = inf.readLine(basicExpressionPattern, "input");
    string pa = ouf.readLine(basicOutputPattern, "output");
    string ja = ans.readLine(basicOutputPattern, "answer");
    if (!isWellformedEquality(in)) {
        quitf(_fail, "input is not well formed: %s", in.c_str());
    }
    if (ja != impossible && ja != correct) {
        if (!isCorrectEquality(ja)) {
            quitf(_fail, "jury answer is not correct: %s", ja.c_str());
        }
        if (!isMovingDigit(in, ja)) {
            quitf(_fail, "jury answer is not moving digit: %s", ja.c_str());
        }
    }
    if (ja == correct) {
        if (!isCorrectEquality(in)) {
            quitf(_fail, "jury inswer is %s but input is not: %s", correct, in.c_str());
        }
    }
    if (pa == impossible) {
        if (ja != impossible) {
            quitf(_wa, "output is %s but answer exists: %s", pa.c_str(), ja.c_str());
        }
        quitf(_ok, "%s", impossible);
    } else if (pa == correct) {
        if (!isCorrectEquality(in)) {
            quitf(_wa, "output is %s but input is not: %s", correct, in.c_str());
        }
        if (ja != correct) {
            quitf(_fail, "output is %s but jury answer is different: %s", correct, ja.c_str());
        }
        quitf(_ok, "%s", correct);
    } else {
        // pa != impossible && pa != correct
        if (!isCorrectEquality(pa)) {
            quitf(_wa, "output is not correct equality: %s", pa.c_str());
        }
        if (!isMovingDigit(in, pa)) {
            quitf(_wa, "output is not moving digit: %s", pa.c_str());
        }
        if (ja == correct) {
            quitf(_wa, "output should be %s but it is moving digits: %s", correct, pa.c_str());
        }
        if (ja == impossible) {
            quitf(_fail, "jury answer is %s but it exists: %s", impossible, pa.c_str());
        }
        quitf(_ok, "%s", pa.c_str());
    }
    quitf(_fail, "Should not reach this line");
    return 0;
}
