#include "testlib.h"
#include <optional>

using namespace std;

typedef long long number;
const number MAX_NUMBER = 9999999999LL;

struct Parser {
    string& s;
    int i;
    int len;

    Parser(string &_s) : s(_s), i(0), len(s.length()) {}

    optional<number> nextNumber() {
        number result = 0;
        int j = i;
        while (j < len && s[j] >= '0' && s[j] <= '9') {
            result = result * 10 + (s[j] - '0');
            if (result > MAX_NUMBER) return nullopt;
            j++;
        }
        if (j == i) return nullopt;
        if (result == 0 && j != i + 1) return nullopt;
        if (result != 0 && s[i] == '0') return nullopt;
        i = j;
        return result;
    }

    optional<number> nextExpression() {
        optional<number> next = nextNumber();
        if (!next) return nullopt;
        number result = next.value();
        while (i < len && (s[i] == '+' || s[i] == '-')) {
            char op = s[i];
            i++;
            next = nextNumber();
            if (!next) return nullopt;
            switch (op) {
                case '+': result += next.value(); break;
                case '-': result -= next.value(); break;
            }
        }
        return result;
    }

    optional<pair<number, number>> nextEquality() {
        optional<number> left = nextExpression();
        if (!left) return nullopt;
        if (i >= len || s[i] != '=') return nullopt;
        i++;
        optional<number> right = nextExpression();
        if (!right) return nullopt;
        if (i != len) return nullopt;
        return make_pair(left.value(), right.value());
    }
};

bool isWellformedEquality(string &s) {
    return Parser(s).nextEquality().has_value();
}

pattern basicExpressionPattern{ "[0-9+\\-]+=[0-9+\\-]+" };

int main(int argc, char* argv[]) {
    registerValidation(argc, argv);
    string in = inf.readLine(basicExpressionPattern, "input");
    ensuref(isWellformedEquality(in), "Must be well formed equality: %s", in.c_str());
    inf.readEof();
    return 0;
}
