#include <random>
#include <vector>
#include <iostream>
#include <unordered_map>
#include <chrono>
#include <functional>
#include <queue>

using uid = std::uniform_int_distribution<int>;
struct Edge {
    int32_t u, v, t;
};

namespace solution1 {
const int32_t MAX_WEIGHT = 1e6;

int32_t find_ans(const int32_t n, const std::vector<Edge>& e) {
    const int32_t entrance = 0, orthanc = n - 1;

    // Dijkstra algorithm in m log n
    const int64_t INF = static_cast<int64_t>(n) * MAX_WEIGHT;
    std::vector<std::vector<int32_t>> ingoing_arc_indices(n);
    for (int32_t arc_index = 0; arc_index < std::ssize(e); ++arc_index) {
        if (e[arc_index].u == n - 1)
            continue;
        ingoing_arc_indices[e[arc_index].v].push_back(arc_index);
    }
    std::vector<int64_t> distance_to_orthanc(n, INF);
    std::priority_queue<std::pair<int64_t, int32_t>, std::vector<std::pair<int64_t, int32_t>>, std::greater<>> nearest;
    nearest.emplace(0, orthanc);
    distance_to_orthanc[orthanc] = 0;
    while (!nearest.empty()) {
        auto [current_distance_to_orthanc, current_tower] = nearest.top();
        nearest.pop();
        if (current_distance_to_orthanc != distance_to_orthanc[current_tower])
            continue;
        for (int32_t arc_index : ingoing_arc_indices[current_tower]) {
            int32_t another_tower = e[arc_index].u;
            int64_t new_distance_to_orthanc = current_distance_to_orthanc + e[arc_index].t;
            if (new_distance_to_orthanc < distance_to_orthanc[another_tower]) {
                distance_to_orthanc[another_tower] = new_distance_to_orthanc;
                nearest.emplace(new_distance_to_orthanc, another_tower);
            }
        }
    }

    // outgoing effective degree
    std::vector<int32_t> outgoing_effective_degree(n);
    for (int32_t arc_index = 0; arc_index < std::ssize(e); ++arc_index)
        if (distance_to_orthanc[e[arc_index].u] == distance_to_orthanc[e[arc_index].v] + e[arc_index].t)
            ++outgoing_effective_degree[e[arc_index].u];

    // bfs with occasional retro-analysis
    std::vector<int32_t> incompetence(n, n);
    std::queue<int32_t> bfs_queue;
    std::queue<int32_t> retro_queue;
    for (int32_t tower_index = 0; tower_index < n; ++tower_index)
        if (distance_to_orthanc[tower_index] == INF) {
            incompetence[tower_index] = -1;
            bfs_queue.push(tower_index);
        }
    while (!bfs_queue.empty() || !retro_queue.empty())
        if (!bfs_queue.empty()) {
            const int32_t current_tower = bfs_queue.front();
            bfs_queue.pop();
            const int32_t new_incompetence = incompetence[current_tower] + 1;
            for (const int32_t arc_index : ingoing_arc_indices[current_tower]) {
                const int32_t another_tower = e[arc_index].u;
                if (incompetence[another_tower] > new_incompetence) {
                    incompetence[another_tower] = new_incompetence;
                    retro_queue.push(another_tower);
                }
            }
        } else
            while (!retro_queue.empty()) {
                const int32_t current_tower = retro_queue.front();
                retro_queue.pop();
                bfs_queue.push(current_tower);
                const int32_t current_incompetence = incompetence[current_tower];
                for (int32_t arc_index : ingoing_arc_indices[current_tower]) {
                    int32_t another_tower = e[arc_index].u;
                    int64_t induced_distance_to_orthanc = distance_to_orthanc[current_tower] + e[arc_index].t;
                    if (induced_distance_to_orthanc != distance_to_orthanc[another_tower])
                        continue;
                    if (incompetence[another_tower] <= current_incompetence)
                        continue;
                    if (--outgoing_effective_degree[another_tower] > 0)
                        continue;
                    incompetence[another_tower] = current_incompetence;
                    retro_queue.push(another_tower);
                }
            }
    return incompetence[entrance];
}
}

namespace solution2 {
const int32_t MAX_WEIGHT = 1e6;

int32_t find_ans(const int32_t n, const std::vector<Edge>& e) {
    const int32_t entrance = 0, orthanc = n - 1;

    // Dijkstra algorithm in m log n
    const int64_t INF = static_cast<int64_t>(n) * MAX_WEIGHT;
    std::vector<std::vector<int32_t>> ingoing_arc_indices(n);
    for (int32_t arc_index = 0; arc_index < std::ssize(e); ++arc_index)
        ingoing_arc_indices[e[arc_index].v].push_back(arc_index);
    std::vector<int64_t> distance_to_orthanc(n, INF);
    std::priority_queue<std::pair<int64_t, int32_t>, std::vector<std::pair<int64_t, int32_t>>, std::greater<>> nearest;
    nearest.emplace(0, orthanc);
    distance_to_orthanc[orthanc] = 0;
    while (!nearest.empty()) {
        auto [current_distance_to_orthanc, current_tower] = nearest.top();
        nearest.pop();
        if (current_distance_to_orthanc != distance_to_orthanc[current_tower])
            continue;
        for (int32_t arc_index : ingoing_arc_indices[current_tower]) {
            int32_t another_tower = e[arc_index].u;
            int64_t new_distance_to_orthanc = current_distance_to_orthanc + e[arc_index].t;
            if (new_distance_to_orthanc < distance_to_orthanc[another_tower]) {
                distance_to_orthanc[another_tower] = new_distance_to_orthanc;
                nearest.emplace(new_distance_to_orthanc, another_tower);
            }
        }
    }

    // find effective edges. Ignore too long ones
    std::vector<std::unordered_map<int32_t, int32_t>> outgoing_arc_indices(n);
    for (int32_t arc_index = 0; arc_index < std::ssize(e); ++arc_index) {
        if (e[arc_index].u == e[arc_index].v)
            continue;
        if (!outgoing_arc_indices[e[arc_index].u].contains(e[arc_index].v)) {
            outgoing_arc_indices[e[arc_index].u][e[arc_index].v] = arc_index;
            continue;
        }
        int32_t& old_arc_index = outgoing_arc_indices[e[arc_index].u][e[arc_index].v];
        if (e[arc_index].t < e[old_arc_index].t)
            old_arc_index = arc_index;
    }

    // recursive search
    std::vector<bool> recurse(n);
    std::function<int32_t(int32_t, int32_t)> find_bounded_incompetence = [&](const int32_t tower, int32_t bound) -> int32_t {
        recurse[tower] = true;
        // no way to Orthanc? Instant loss.
        if (distance_to_orthanc[tower] == INF) {
            recurse[tower] = false;
            return -1;
        }
        // already in Orthanc? Instant win.
        if (tower == n - 1) {
            recurse[tower] = false;
            return bound;
        }
        int32_t best_effective = -1;
        // let us try and spend some incompetence
        for (const auto [to, arc_index] : outgoing_arc_indices[tower]) {
            // incompetence is over? Instant win.
            if (bound <= 0) {
                recurse[tower] = false;
                return 0;
            }
            // if already in our path, not worth going in
            if (recurse[to]) {
                if (distance_to_orthanc[e[arc_index].u] == distance_to_orthanc[e[arc_index].v] + e[arc_index].t)
                    best_effective = bound;
                continue;
            }
            // if bad edge, have to spend some incompetence
            int32_t incompetence_delta = 0;
            if (distance_to_orthanc[e[arc_index].u] != distance_to_orthanc[e[arc_index].v] + e[arc_index].t)
                incompetence_delta = 1;
            int32_t remaining_incompetence = bound - incompetence_delta;
            remaining_incompetence = find_bounded_incompetence(to, remaining_incompetence);
            if (remaining_incompetence + 1 < bound)
                bound = remaining_incompetence + 1;
            if (incompetence_delta == 1)
                continue;
            if (remaining_incompetence >= best_effective)
                best_effective = remaining_incompetence;
        }
        recurse[tower] = false;
        return std::min(bound, best_effective);
    };
    return find_bounded_incompetence(entrance, n);
}
}

void stress() {
    static std::mt19937 rng;
    int32_t MAX_N = 1000, MAX_M = 100;
    int32_t MAX_S = MAX_N + MAX_M;
    int32_t MAX_MAX_T = 1000000;
    int64_t tries = 0;
    std::chrono::high_resolution_clock::duration sol1dur{}, sol2dur{};
    while (true) {
        int32_t n = uid(2, MAX_N)(rng);
        int32_t m = uid(2, MAX_M)(rng);
        int32_t max_t = uid(1, MAX_MAX_T)(rng);
        if (n + m > MAX_S)
            continue;
        ++tries;
        std::vector<Edge> e(m);
        for (Edge& f : e) {
            f.u = uid(0, n - 1)(rng);
            f.v = uid(0, n - 1)(rng);
            f.t = uid(1, max_t)(rng);
        }
        std::chrono::high_resolution_clock::time_point t1 = std::chrono::high_resolution_clock::now();
        int32_t ans = solution1::find_ans(n, e);
        std::chrono::high_resolution_clock::time_point t2 = std::chrono::high_resolution_clock::now();
        int32_t other_ans = solution2::find_ans(n, e);
        std::chrono::high_resolution_clock::time_point t3 = std::chrono::high_resolution_clock::now();
        sol1dur += t2 - t1;
        sol2dur += t3 - t2;
        if (ans != other_ans) {
            std::cout << n << ' ' << e.size() << '\n';
            for (const Edge& f : e)
                std::cout << f.u + 1 << ' ' << f.v + 1 << ' ' << f.t << '\n';
            std::cout << "sol1: " << ans << '\n';
            std::cout << "sol2: " << other_ans << '\n';
            MAX_S = n + m - 1;
            MAX_N = std::min(MAX_N, MAX_S);
            MAX_M = std::min(MAX_M, MAX_S);
            MAX_MAX_T = max_t;
            tries = 0;
            continue;
        }
        if ((tries & tries - 1) == 0) {
            std::cout << std::format("TRIES {}: sol1 {}, sol2 {}\n", tries, sol1dur / tries, sol2dur / tries);
        }
    }
}

int main() {
    stress();
}
