#include <random>
#include <vector>
#include <iostream>
#include <unordered_map>
#include <chrono>
#include <functional>
#include <queue>
#include <set>
#include <format>

using uid = std::uniform_int_distribution<int>;
struct Edge {
    int32_t u, v, t;
};

namespace solution1 {
const int32_t MAX_WEIGHT = 1e6;

int32_t find_ans(const int32_t n, const std::vector<Edge>& e) {
    const int32_t entrance = 0, orthanc = n - 1;

    // Dijkstra algorithm in m log n
    const int64_t INF = static_cast<int64_t>(n) * MAX_WEIGHT;
    std::vector<std::vector<int32_t>> ingoing_arc_indices(n);
    for (int32_t arc_index = 0; arc_index < std::ssize(e); ++arc_index) {
        if (e[arc_index].u == n - 1)
            continue;
        ingoing_arc_indices[e[arc_index].v].push_back(arc_index);
    }
    std::vector<int64_t> distance_to_orthanc(n, INF);
    std::priority_queue<std::pair<int64_t, int32_t>, std::vector<std::pair<int64_t, int32_t>>, std::greater<>> nearest;
    nearest.emplace(0, orthanc);
    distance_to_orthanc[orthanc] = 0;
    while (!nearest.empty()) {
        auto [current_distance_to_orthanc, current_tower] = nearest.top();
        nearest.pop();
        if (current_distance_to_orthanc != distance_to_orthanc[current_tower])
            continue;
        for (int32_t arc_index : ingoing_arc_indices[current_tower]) {
            int32_t another_tower = e[arc_index].u;
            int64_t new_distance_to_orthanc = current_distance_to_orthanc + e[arc_index].t;
            if (new_distance_to_orthanc < distance_to_orthanc[another_tower]) {
                distance_to_orthanc[another_tower] = new_distance_to_orthanc;
                nearest.emplace(new_distance_to_orthanc, another_tower);
            }
        }
    }

    // outgoing effective degree
    std::vector<int32_t> outgoing_effective_degree(n);
    for (int32_t arc_index = 0; arc_index < std::ssize(e); ++arc_index)
        if (distance_to_orthanc[e[arc_index].u] == distance_to_orthanc[e[arc_index].v] + e[arc_index].t)
            ++outgoing_effective_degree[e[arc_index].u];

    // bfs with occasional retro-analysis
    std::vector<int32_t> incompetence(n, n);
    std::queue<int32_t> bfs_queue;
    std::queue<int32_t> retro_queue;
    for (int32_t tower_index = 0; tower_index < n; ++tower_index)
        if (distance_to_orthanc[tower_index] == INF) {
            incompetence[tower_index] = -1;
            bfs_queue.push(tower_index);
        }
    while (!bfs_queue.empty() || !retro_queue.empty())
        if (!bfs_queue.empty()) {
            const int32_t current_tower = bfs_queue.front();
            bfs_queue.pop();
            const int32_t new_incompetence = incompetence[current_tower] + 1;
            for (const int32_t arc_index : ingoing_arc_indices[current_tower]) {
                const int32_t another_tower = e[arc_index].u;
                if (incompetence[another_tower] > new_incompetence) {
                    incompetence[another_tower] = new_incompetence;
                    retro_queue.push(another_tower);
                }
            }
        } else
            while (!retro_queue.empty()) {
                const int32_t current_tower = retro_queue.front();
                retro_queue.pop();
                bfs_queue.push(current_tower);
                const int32_t current_incompetence = incompetence[current_tower];
                for (int32_t arc_index : ingoing_arc_indices[current_tower]) {
                    int32_t another_tower = e[arc_index].u;
                    int64_t induced_distance_to_orthanc = distance_to_orthanc[current_tower] + e[arc_index].t;
                    if (induced_distance_to_orthanc != distance_to_orthanc[another_tower])
                        continue;
                    if (incompetence[another_tower] <= current_incompetence)
                        continue;
                    if (--outgoing_effective_degree[another_tower] > 0)
                        continue;
                    incompetence[another_tower] = current_incompetence;
                    retro_queue.push(another_tower);
                }
            }
    return incompetence[entrance];
}
}

namespace solution2 {
using namespace std;
const int N = 500005;
const int inf = 1000 * 1000 * 1000;
const int mod = 998244353;
// mt19937 myrand(chrono::steady_clock::now().time_since_epoch().count());
mt19937 myrand(373);

int n, m;

vector < pair<int, int> > g[N], gr[N];

void clean() {
    for (int i = 1; i <= n; ++i) {
        g[i].clear();
        gr[i].clear();
    }
}

int32_t find_ans(const int32_t input_n, const std::vector<Edge>& e) {
    n = input_n;
    m = e.size();

    for (const Edge& f : e) {
        g[f.u + 1].emplace_back(f.v + 1, f.t);
        gr[f.v + 1].emplace_back(f.u + 1, f.t);
    }

    vector <bool> mark(n + 1, false);
    queue <int> q;
    q.push(n);
    mark[n] = true;

    while (!q.empty()) {
        int v = q.front();
        q.pop();

        for (auto [to, _]: gr[v]) {
            if (mark[to]) continue;
            mark[to] = true;
            q.push(to);
        }
    }

    vector <int> dist(n + 1, inf);
    for (int i = 1; i <= n; i++) {
        if (!mark[i]) {
            dist[i] = 0;
            q.push(i);
        }
    }

    while (!q.empty()) {
        int v = q.front();
        q.pop();

        for (auto [to, _]: gr[v]) {
            if (dist[to] != inf || to == n) continue;
            dist[to] = dist[v] + 1;
            q.push(to);
        }
    }

    vector <long long> reverse_dist(n + 1, inf * 1ll * inf);
    set < pair<long long, int> > Q;
    reverse_dist[n] = 0;
    Q.insert(make_pair(reverse_dist[n], n));

    while (!Q.empty()) {
        int v = Q.begin() -> second;
        Q.erase(Q.begin());

        for (auto [to, len]: gr[v]) {
            if (reverse_dist[v] + len < reverse_dist[to]) {
                Q.erase(make_pair(reverse_dist[to], to));
                reverse_dist[to] = reverse_dist[v] + len;
                Q.insert(make_pair(reverse_dist[to], to));
            }
        }
    }

    // for (int i = 1; i <= n; i++) {
    //   cout << dist[i] << " ";
    // }
    // cout << endl;

    // for (int i = 1; i <= n; i++) {
    //   cout << reverse_dist[i] << " ";
    // }
    // cout << endl;

    // for (int i = 1; i <= n; i++) {
    //   cout << mark[i] << " ";
    // }
    // cout << endl;

    if (reverse_dist[1] == inf * 1ll * inf) {
        clean();
        return -1;
    }

    int ina = 1, inb = n, answ = 0;
    while (ina <= inb) {
        int mid = (ina + inb) / 2;

        if (dist[1] <= mid) {
            inb = mid - 1;
            continue;
        }

        vector <bool> used(n + 1, false);
        queue <int> q;
        used[1] = true;
        q.push(1);

        while (!q.empty()) {
            int v = q.front();
            q.pop();

            for (auto [to, len]: g[v]) {
                if (dist[to] <= mid) continue;
                if (reverse_dist[v] != len + reverse_dist[to]) continue;
                if (used[to]) continue;
                used[to] = true;
                q.push(to);
            }
        }

        if (used[n]) {
            answ = mid;
            ina = mid + 1;
        } else {
            inb = mid - 1;
        }
    }

    clean();
    return answ;
}

}

void stress() {
    static std::mt19937 rng;
    int32_t MAX_N = 1000, MAX_M = 100;
    int32_t MAX_S = MAX_N + MAX_M;
    int32_t MAX_MAX_T = 1000000;
    int64_t tries = 0;
    std::chrono::high_resolution_clock::duration sol1dur{}, sol2dur{};
    while (true) {
        int32_t n = uid(2, MAX_N)(rng);
        int32_t m = uid(2, MAX_M)(rng);
        int32_t max_t = uid(1, MAX_MAX_T)(rng);
        if (n + m > MAX_S)
            continue;
        ++tries;
        std::vector<Edge> e(m);
        for (Edge& f : e) {
            f.u = uid(0, n - 1)(rng);
            f.v = uid(0, n - 1)(rng);
            f.t = uid(1, max_t)(rng);
        }
        std::chrono::high_resolution_clock::time_point t1 = std::chrono::high_resolution_clock::now();
        int32_t ans = solution1::find_ans(n, e);
        std::chrono::high_resolution_clock::time_point t2 = std::chrono::high_resolution_clock::now();
        int32_t other_ans = solution2::find_ans(n, e);
        std::chrono::high_resolution_clock::time_point t3 = std::chrono::high_resolution_clock::now();
        sol1dur += t2 - t1;
        sol2dur += t3 - t2;
        if (ans != other_ans) {
            std::cout << n << ' ' << e.size() << '\n';
            for (const Edge& f : e)
                std::cout << f.u + 1 << ' ' << f.v + 1 << ' ' << f.t << '\n';
            std::cout << "sol1: " << ans << '\n';
            std::cout << "sol2: " << other_ans << '\n';
            MAX_S = n + m - 1;
            MAX_N = std::min(MAX_N, MAX_S);
            MAX_M = std::min(MAX_M, MAX_S);
            MAX_MAX_T = max_t;
            tries = 0;
            continue;
        }
        if ((tries & tries - 1) == 0) {
            std::cout << std::format("TRIES {}: sol1 {}, sol2 {}\n", tries, sol1dur / tries, sol2dur / tries);
        }
    }
}

int main() {
    stress();
}
