#include <bits/stdc++.h>

/**
 * Author: Niyaz Nigmatullin
 *
 */

using namespace std;

string to_string(string s) {
  return '"' + s + '"';
}

string to_string(const char* s) {
  return to_string((string) s);
}

string to_string(bool b) {
  return (b ? "true" : "false");
}

template <typename A, typename B>
string to_string(pair<A, B> p) {
  return "(" + to_string(p.first) + ", " + to_string(p.second) + ")";
}

template <typename A>
string to_string(A v) {
  bool first = true;
  string res = "{";
  for (const auto &x : v) {
    if (!first) {
      res += ", ";
    }
    first = false;
    res += to_string(x);
  }
  res += "}";
  return res;
}

void debug_out() { cerr << endl; }

template <typename Head, typename... Tail>
void debug_out(Head H, Tail... T) {
  cerr << " " << to_string(H);
  debug_out(T...);
}

#ifdef LOCAL
#define debug(...) cerr << "[" << #__VA_ARGS__ << "]:", debug_out(__VA_ARGS__)
#else
#define debug(...) 42
#endif


// https://pastebin.com/exQM152L

template <typename T>
class flow_graph {
  public:
  static constexpr T eps = (T) 1e-9;

  struct edge {
    int to;
    T c;
    T f;
    int rev;
  };

  vector< vector<edge> > g;
  vector<int> ptr;
  vector<int> d;
  vector<int> q;
  vector<int> cnt_on_layer;
  vector<int> prev_edge;
  bool can_reach_sink;

  int n;
  int st, fin;
  T flow;
   
  flow_graph(int _n, int _st, int _fin) : n(_n), st(_st), fin(_fin) {
    assert(0 <= st && st < n && 0 <= fin && fin < n && st != fin);
    g.resize(n);
    ptr.resize(n);
    d.resize(n);
    q.resize(n);
    cnt_on_layer.resize(n + 1);
    prev_edge.resize(n);
    flow = 0;
  }

  void clear_flow() {
    for (int i = 0; i < n; i++) {
      for (edge &e : g[i]) {
        e.f = 0;
      }
    }
    flow = 0;
  }
   
  int add(int from, int to, T forward_cap, T backward_cap) {
    assert(0 <= from && from < n && 0 <= to && to < n);
    int from_size = (int) g[from].size();
    int to_size = (int) g[to].size();
    g[from].push_back({to, forward_cap, 0, to_size});
    g[to].push_back({from, backward_cap, 0, from_size});
    return from_size;
  }

  bool expath() {
    fill(d.begin(), d.end(), n);
    q[0] = fin;
    d[fin] = 0;
    fill(cnt_on_layer.begin(), cnt_on_layer.end(), 0);
    cnt_on_layer[n] = n - 1;
    cnt_on_layer[0] = 1;
    int beg = 0, end = 1;
    while (beg < end) {
      int i = q[beg++];
      for (const edge &e : g[i]) {
        const edge &back = g[e.to][e.rev];
        if (back.c - back.f > eps && d[e.to] == n) {
          cnt_on_layer[d[e.to]]--;
          d[e.to] = d[i] + 1;
          cnt_on_layer[d[e.to]]++;
          q[end++] = e.to;
        }
      }
    }
    return (d[st] != n);
  }

  T augment(int &v) {
    T cur = numeric_limits<T>::max();
    int i = fin;
    while (i != st) {
      const edge &e = g[i][prev_edge[i]];
      const edge &back = g[e.to][e.rev];
      cur = min(cur, back.c - back.f);
      i = e.to;
    }
    i = fin;
    while (i != st) {
      edge &e = g[i][prev_edge[i]];
      edge &back = g[e.to][e.rev];
      back.f += cur;
      e.f -= cur;
      i = e.to;
      if (back.c - back.f <= eps) {
        v = i;
      }
    }
    return cur;
  }

  int retreat(int v) {
    int new_dist = n - 1;
    for (const edge &e : g[v]) {
      if (e.c - e.f > eps && d[e.to] < new_dist) {
        new_dist = d[e.to];
      }
    }
    cnt_on_layer[d[v]]--;
    if (cnt_on_layer[d[v]] == 0) {
      if (new_dist + 1 > d[v]) {
        can_reach_sink = false;
      }
    }
    d[v] = new_dist + 1;
    cnt_on_layer[d[v]]++;
    if (v != st) {
      v = g[v][prev_edge[v]].to;
    }
    return v;
  }
   
  T max_flow() {
    can_reach_sink = true;
    for (int i = 0; i < n; i++) {
      ptr[i] = (int) g[i].size() - 1;
    }
    if (expath()) {
      int v = st;
      while (d[st] < n) {
        while (ptr[v] >= 0) {
          const edge &e = g[v][ptr[v]];
          if (e.c - e.f > eps && d[e.to] == d[v] - 1) {
            prev_edge[e.to] = e.rev;
            v = e.to;
            if (v == fin) {
              flow += augment(v);
            }
            break;
          }
          ptr[v]--;
        }
        if (ptr[v] < 0) {
          ptr[v] = (int) g[v].size() - 1;
          v = retreat(v);
          if (!can_reach_sink) {
            break;
          }
        }
      }
    }
    return flow;
  }

  vector<bool> min_cut() {
    max_flow();
    assert(!expath());
    vector<bool> ret(n);
    for (int i = 0; i < n; i++) {
      ret[i] = (d[i] != n);
    }
    return ret;
  }
};


struct Block {
    int start;
    int end;
    int count;

    Block(int s, int e, int c) : start(s), end(e), count(c) {}
};

std::optional<std::vector<Block>> buildBlocks(const std::vector<int>& inv, const std::vector<int>& d) {
    int n = (int) inv.size();
    std::vector<std::pair<int, int>> elements;

    // Build initial list of index-value pairs
    elements.emplace_back(-1, -1);
    for (size_t i = 0; i < d.size(); ++i) {
        if (d[i] >= 0) {
            elements.emplace_back(i, inv[d[i]]);
        }
    }
    elements.emplace_back(d.size(), n);

    std::vector<Block> blocks;

    for (size_t i = 1; i < elements.size(); ++i) {
        const auto& left = elements[i - 1];
        const auto& right = elements[i];

        blocks.emplace_back(left.second + 1, right.second, right.first - left.first - 1);

        if (left.second > right.second) {
            return std::nullopt; // Equivalent to returning null in Kotlin
        }

        if (right.second >= 0 && right.second < n) {
            blocks.emplace_back(right.second, right.second + 1, 1);
        }
    }

    return blocks;
}

std::vector<int> buildBlockWhere(const std::vector<int>& p, const std::vector<Block>& blocks) {
    int n = (int) p.size();
    std::vector<int> where(n, -1); // Initialize a vector of size `n` with all elements as -1.

    for (int blockID = 0; blockID < (int) blocks.size(); ++blockID) {
        const Block& b = blocks[blockID];
        for (int index = b.start; index < b.end; ++index) {
            where[p[index]] = blockID; // Assign the block ID to the `where` array.
        }
    }

    return where;
}

vector<int> readSubOne(int n) {
	vector<int> res(n);
	for (auto &e: res) {
		cin >> e;
		if (e >= 0) e--;
	}
	return res;
}

enum class Solution {
    NO_WAY,
    MAYBE,
    HAS_TO
};

std::vector<int> inv(const std::vector<int>& p) {
    int n = (int) p.size();
    std::vector<int> inverse(n);
    for (int i = 0; i < n; ++i) {
        inverse[p[i]] = i;
    }
    return inverse;
}

std::optional<std::vector<Solution>> solve(
    const std::vector<int>& p1,
    const std::vector<int>& p2,
    const std::vector<int>& d1,
    const std::vector<int>& d2
) {
    int n = (int) p1.size();
    std::vector<int> inv1 = inv(p1);
    std::vector<int> inv2 = inv(p2);
    auto blocks1_opt = buildBlocks(inv1, d1);
    if (!blocks1_opt.has_value()) return std::nullopt;
    auto blocks2_opt = buildBlocks(inv2, d2);
    if (!blocks2_opt.has_value()) return std::nullopt;
    std::vector<Block> blocks1 = blocks1_opt.value();
    std::vector<Block> blocks2 = blocks2_opt.value();
    int m = (int) d1.size();
    assert(std::accumulate(blocks1.begin(), blocks1.end(), 0, [](int sum, const Block& b) {
        return sum + b.count;
    }) == m);

    assert(std::accumulate(blocks2.begin(), blocks2.end(), 0, [](int sum, const Block& b) {
        return sum + b.count;
    }) == m);

    std::vector<int> where1 = buildBlockWhere(p1, blocks1);
    std::vector<int> where2 = buildBlockWhere(p2, blocks2);

    int source = (int) (blocks1.size() + blocks2.size());
    int sink = source + 1;
    flow_graph<int> g(sink + 1, source, sink);
    vector<int> edgesB1(blocks1.size());
    for (int i = 0; i < (int) blocks1.size(); i++) {
    	edgesB1[i] = g.add(source, i, blocks1[i].count, 0);
    }
    vector<int> edgesB2(blocks2.size());
    for (int i = 0; i < (int) blocks2.size(); i++) {
    	edgesB2[i] = g.add((int) blocks1.size() + i, sink, blocks2[i].count, 0);
    }

    vector<int> edges(n);

    for (int i = 0; i < n; i++) {
    	edges[i] = g.add(where1[i], (int) blocks1.size() + where2[i], 1, 0);
    }

    if (g.max_flow() != m) {
    	return nullopt;
    }
    g.clear_flow();

    std::vector<Solution> ans(n, Solution::NO_WAY);

    auto can_without = [&](int x) {
    	g.g[where1[x]][edges[x]].c = 0;
    	int flow = g.max_flow();
    	g.clear_flow();
    	g.g[where1[x]][edges[x]].c = 1;
    	return flow == m;
    };

    auto can_with = [&](int x) {
    	auto &e1 = g.g[source][edgesB1[where1[x]]];
    	auto &e2 = g.g[where2[x] + (int) blocks1.size()][edgesB2[where2[x]]];
    	if (e1.c > 0 && e2.c > 0) {
    		e1.c--;
    		e2.c--;
    		g.g[where1[x]][edges[x]].c = 0;
    		int flow = g.max_flow();
    		g.clear_flow();
    		e1.c++;
    		e2.c++;
    		g.g[where1[x]][edges[x]].c = 1;
    		return flow == m - 1;
    	} else return false;
    };
    for (int i = 0; i < n; i++) {
    	bool without = can_without(i);
    	bool with = can_with(i);
    	assert(without || with);
    	if (without && with) {
    		ans[i] = Solution::MAYBE;
    	} else if (without) {
    		ans[i] = Solution::NO_WAY;
    	} else {
    		ans[i] = Solution::HAS_TO;
    	}
    }

    return ans;
}

void solveTest() {
	int n;
	cin >> n;
	auto p1 = readSubOne(n);
	auto p2 = readSubOne(n);
	int m;
	cin >> m;
	auto d1 = readSubOne(m);
	auto d2 = readSubOne(m);
	auto ans = solve(p1, p2, d1, d2);
	if (!ans.has_value()) {
		cout << "Inconsistent\n";
		return;
	}
	for (auto &e: ans.value()) {
		cout << "N?Y"[(int) e];
	}
	cout << '\n';
}


int main() {
	std::cin.tie(NULL); std::ios::sync_with_stdio(false);
	int t;
	cin >> t;
	while (t--) solveTest();
}
